#define ONFRONTCONNECTED 0
#define ONFRONTDISCONNECTED 1
#define ONHEARTBEATWARNING 2
#define ONRSPAUTHENTICATE 3
#define ONRSPUSERLOGIN 4
#define ONRSPUSERLOGOUT 5
#define ONRSPORDERINSERT 6
#define ONRSPORDERACTION 7
#define ONRSPEXECORDERINSERT 8
#define ONRSPEXECORDERACTION 9
#define ONRSPFORQUOTEINSERT 10
#define ONRSPQUOTEINSERT 11
#define ONRSPQUOTEACTION 12
#define ONRSPBATCHORDERACTION 13
#define ONRSPOPTIONSELFCLOSEINSERT 14
#define ONRSPOPTIONSELFCLOSEACTION 15
#define ONRSPCOMBACTIONINSERT 16
#define ONRSPQRYORDER 17
#define ONRSPQRYTRADE 18
#define ONRSPQRYINVESTORPOSITION 19
#define ONRSPQRYTRADINGACCOUNT 20
#define ONRSPQRYINVESTOR 21
#define ONRSPQRYTRADINGCODE 22
#define ONRSPQRYINSTRUMENTMARGINRATE 23
#define ONRSPQRYINSTRUMENTCOMMISSIONRATE 24
#define ONRSPQRYEXCHANGE 25
#define ONRSPQRYPRODUCT 26
#define ONRSPQRYINSTRUMENT 27
#define ONRSPQRYCOMBINSTRUMENT 28
#define ONRSPQRYCOMBACTION 29
#define ONRSPQRYINVESTORPOSITIONFORCOMB 30
#define ONRSPQRYDEPTHMARKETDATA 31
#define ONRSPQRYINSTRUMENTSTATUS 32
#define ONRSPQRYINVESTORPOSITIONDETAIL 33
#define ONRSPQRYINVESTORPOSITIONCOMBINEDETAIL 34
#define ONRSPQRYEXCHANGEMARGINRATE 35
#define ONRSPQRYEXCHANGEMARGINRATEADJUST 36
#define ONRSPQRYOPTIONINSTRTRADECOST 37
#define ONRSPQRYOPTIONINSTRCOMMRATE 38
#define ONRSPQRYEXECORDER 39
#define ONRSPQRYFORQUOTE 40
#define ONRSPQRYFORQUOTEPARAM 41
#define ONRSPQRYQUOTE 42
#define ONRSPQRYOPTIONSELFCLOSE 43
#define ONRSPERROR 44
#define ONRTNORDER 45
#define ONRTNTRADE 46
#define ONERRRTNORDERINSERT 47
#define ONERRRTNORDERACTION 48
#define ONRTNINSTRUMENTSTATUS 49
#define ONRTNEXECORDER 50
#define ONERRRTNEXECORDERINSERT 51
#define ONERRRTNEXECORDERACTION 52
#define ONERRRTNFORQUOTEINSERT 53
#define ONRTNQUOTE 54
#define ONERRRTNQUOTEINSERT 55
#define ONERRRTNQUOTEACTION 56
#define ONRTNFORQUOTERSP 57
#define ONERRRTNBATCHORDERACTION 58
#define ONRTNOPTIONSELFCLOSE 59
#define ONERRRTNOPTIONSELFCLOSEINSERT 60
#define ONERRRTNOPTIONSELFCLOSEACTION 61
#define ONRTNCOMBACTION 62
#define ONRSPQRYINSTRUMENTORDERCOMMRATE 63
